#include "thread/Clock.h"
#include "arch/x86_64/IO.h"

// Every timer tick is equivalent to 250 microseconds.
// FIXME: Change ARCH_TIMER_RESOLUTION to use nanoseconds.
const usize ARCH_TIMER_RESOLUTION = 250;

#define PIT_CHANNEL_0 0x40

const u64 base_frequency = 1193182;

void Clock::arch_init()
{
    constexpr u16 divisor = (u16)(base_frequency / ((1000 / ARCH_TIMER_RESOLUTION) * 1000));
    static_assert(divisor >= 100, "ARCH_TIMER_RESOLUTION is too low");
    IO::outb(PIT_CHANNEL_0, (u8)(divisor & 0xFF));
    IO::outb(0x80, 0); // short delay
    IO::outb(PIT_CHANNEL_0, (u8)((divisor & 0xFF00) >> 8));

    long resolution = ARCH_TIMER_RESOLUTION * 1000;

    g_realtime_clock.set_resolution(resolution);
    g_monotonic_clock.set_resolution(resolution);
}
